#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from .GXDLMSObject import GXDLMSObject
from .IGXDLMSBase import IGXDLMSBase
from ..enums import ErrorCode
from ..internal._GXCommon import _GXCommon
from ..GXByteBuffer import GXByteBuffer
from ..enums import ObjectType, DataType
from .enums import BaudRate
from .GXDLMSModemInitialisation import GXDLMSModemInitialisation

# pylint: disable=too-many-instance-attributes
class GXDLMSModemConfiguration(GXDLMSObject, IGXDLMSBase):
    """
    Online help:
    http://www.gurux.fi/Gurux.DLMS.Objects.GXDLMSModemConfiguration
    """

    #
    # Constructor.
    #
    # @param ln
    # Logical Name of the object.
    # @param sn
    # Short Name of the object.
    #
    def __init__(self, ln=None, sn=0):
        super(GXDLMSModemConfiguration, self).__init__(ObjectType.MODEM_CONFIGURATION, ln, sn)
        self.initialisationStrings = list()
        self.communicationSpeed = BaudRate.BAUDRATE_300
        self.modemProfile = self.defultProfiles()

    @classmethod
    def defultProfiles(cls):
        return ["OK", "CONNECT", "RING", "NO CARRIER", "ERROR",
                "CONNECT 1200", "NO DIAL TONE", "BUSY", "NO ANSWER",
                "CONNECT 600", "CONNECT 2400", "CONNECT 4800", "CONNECT 9600",
                "CONNECT 14 400", "CONNECT 28 800", "CONNECT 33 600",
                "CONNECT 56 000"]

    def getValues(self):
        return [self.logicalName,
                self.communicationSpeed,
                self.initialisationStrings,
                self.modemProfile]

    #
    # Returns collection of attributes to read.  If attribute is static
    # and already read or device is returned HW error it is not returned.
    def getAttributeIndexToRead(self, all_):
        attributes = list()
        #  LN is static and read only once.
        if all_ or not self.logicalName:
            attributes.append(1)
        #  CommunicationSpeed
        if all_ or not self.isRead(2):
            attributes.append(2)
        #  InitialisationStrings
        if all_ or not self.isRead(3):
            attributes.append(3)
        #  ModemProfile
        if all_ or not self.isRead(4):
            attributes.append(4)
        return attributes

    #
    # Returns amount of attributes.
    #
    def getAttributeCount(self):
        return 4

    #
    # Returns amount of methods.
    #
    def getMethodCount(self):
        return 0

    def getDataType(self, index):
        if index == 1:
            return DataType.OCTET_STRING
        if index == 2:
            return DataType.ENUM
        if index == 3:
            return DataType.ARRAY
        if index == 4:
            return DataType.ARRAY
        raise ValueError("getDataType failed. Invalid attribute index.")

    #
    # Returns value of given attribute.
    #
    def getValue(self, settings, e):
        if e.index == 1:
            return _GXCommon.logicalNameToBytes(self.logicalName)
        if e.index == 2:
            return self.communicationSpeed
        if e.index == 3:
            data = GXByteBuffer()
            data.setUInt8(DataType.ARRAY)
            #  Add count
            cnt = 0
            if self.initialisationStrings:
                cnt = len(self.initialisationStrings)
            _GXCommon.setObjectCount(cnt, data)
            if cnt != 0:
                for it in self.initialisationStrings:
                    data.setUInt8(DataType.STRUCTURE)
                    data.setUInt8(3)
                    #  Count
                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it.request))
                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it.response))
                    _GXCommon.setData(settings, data, DataType.UINT16, it.delay)
            return data
        if e.index == 4:
            data = GXByteBuffer()
            data.setUInt8(DataType.ARRAY)
            #  Add count
            cnt = len(self.modemProfile)
            _GXCommon.setObjectCount(cnt, data)
            if cnt != 0:
                for it in self.modemProfile:
                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, _GXCommon.getBytes(it))
            return data
        e.error = ErrorCode.READ_WRITE_DENIED
        return None

    #
    # Set value of given attribute.
    #
    def setValue(self, settings, e):
        if e.index == 1:
            self.logicalName = _GXCommon.toLogicalName(e.value)
        elif e.index == 2:
            self.communicationSpeed = e.value
        elif e.index == 3:
            self.initialisationStrings = []
            if e.value:
                for it in e.value:
                    item = GXDLMSModemInitialisation()
                    item.request = it[0].decode("utf-8")
                    item.response = it[1].decode("utf-8")
                    if len(it) == 3:
                        item.delay = it[2]
                    self.initialisationStrings.append(item)
        elif e.index == 4:
            self.modemProfile = []
            if e.value:
                for it in e.value:
                    self.modemProfile.append(it.decode("utf-8"))
        else:
            e.error = ErrorCode.READ_WRITE_DENIED

    def load(self, reader):
        self.communicationSpeed = BaudRate(reader.readElementContentAsInt("CommunicationSpeed"))
        if reader.isStartElement("InitialisationStrings", True):
            while reader.isStartElement("Initialisation", True):
                it = GXDLMSModemInitialisation()
                it.request = reader.readElementContentAsString("Request")
                it.response = reader.readElementContentAsString("Response")
                it.delay = reader.readElementContentAsInt("Delay")
            reader.readEndElement("InitialisationStrings")
        self.modemProfile = reader.readElementContentAsString("ModemProfile", "").split(';')

    def save(self, writer):
        writer.writeElementString("CommunicationSpeed", int(self.communicationSpeed))
        writer.writeStartElement("InitialisationStrings")
        if self.initialisationStrings:
            for it in self.initialisationStrings:
                writer.writeStartElement("Initialisation")
                writer.writeElementString("Request", it.request)
                writer.writeElementString("Response", it.response)
                writer.writeElementString("Delay", it.delay)
                writer.writeEndElement()
        writer.writeEndElement()
        if self.modemProfile:
            writer.writeElementString("ModemProfile", ';'.join(self.modemProfile))



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
class SerialNumberCounter:
    @classmethod
    def __getValues(cls, expressions):
        """
        Get values to count together.
        """
        values = list()
        last = 0
        index = 0
        for ch in expressions:
            if ch in ('%', '+', '-', '*', '/'):
                values.append(expressions[last: index])
                values.append(str(ch))
                last = index + 1
            index += 1
        if index != last:
            values.append(expressions[last:])
        return values

    @classmethod
    def __getValue(cls, value, sn):
        if value == "sn":
            return sn
        return int(value)

    # Count serial number using formula.
    # sn: Serial number
    # formula: Formula to used.
    @classmethod
    def count(cls, sn, formula):
        values = cls.__getValues(cls.__formatString(formula))
        if len(values) % 2 == 0:
            raise ValueError("Invalid serial number formula.")
        str_ = None
        value = cls.__getValue(values[0], sn)
        index = 1
        while index != len(values):
            str_ = values[index]
            if str_ == "%":
                value = value % cls.__getValue(values[index + 1], sn)
            elif str_ == "+":
                value = value + cls.__getValue(values[index + 1], sn)
            elif str_ == "-":
                value = value - cls.__getValue(values[index + 1], sn)
            elif str_ == "*":
                value = value * cls.__getValue(values[index + 1], sn)
            elif str_ == "/":
                value = value / cls.__getValue(values[index + 1], sn)
            else:
                raise ValueError("Invalid serial number formula.")
            index += 2
        return value

    #
    # Produce formatted String by the given math expression.
    #
    # @param expression
    #            Unformatted math expression.
    # Formatted math expression.
    #
    @classmethod
    def __formatString(cls, expression):
        if not expression:
            raise ValueError("Expression is null or empty")
        sb = ""
        for ch in expression.lower():
            if ch in ('(', ')'):
                raise ValueError("Invalid serial number formula.")
            #  Is not white space.
            if ch != ' ':
                sb += ch
        return sb



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from __future__ import print_function
from .GXDLMSObject import GXDLMSObject
from .IGXDLMSBase import IGXDLMSBase
from ..internal._GXCommon import _GXCommon
from ..GXByteBuffer import GXByteBuffer
from ..enums import ErrorCode, ObjectType, DataType
from .enums import SecuritySuite, CertificateEntity, CertificateType, SecurityPolicy0, SecurityPolicy
from .GXDLMSCertificateInfo import GXDLMSCertificateInfo
from ..enums.Security import Security
from .enums.GlobalKeyType import GlobalKeyType

# pylint: disable=too-many-public-methods,too-many-instance-attributes
class GXDLMSSecuritySetup(GXDLMSObject, IGXDLMSBase):
    """
    Online help:
    http://www.gurux.fi/Gurux.DLMS.Objects.GXDLMSSecuritySetup
    """
    def __init__(self, ln="0.0.43.0.0.255", sn=0):
        """
        Constructor.

        ln : Logical Name of the object.
        sn : Short Name of the object.
        """
        super(GXDLMSSecuritySetup, self).__init__(ObjectType.SECURITY_SETUP, ln, sn)
        self.securityPolicy = SecurityPolicy0.NOTHING
        # Security policy for version 1.
        self.securitySuite = SecuritySuite.AES_GCM_128
        self.certificates = list()
        # Client system title.
        self.clientSystemTitle = None
        # Server system title.
        self.serverSystemTitle = None
        # Available certificates.
        self.certificates = list()

    def getValues(self):
        return [self.logicalName,
                self.securityPolicy,
                self.securitySuite,
                self.clientSystemTitle,
                self.serverSystemTitle,
                self.certificates]

    #
    # Activates and strengthens the security policy.
    #
    # client: DLMS client that is used to generate action.
    # security: New security level.
    # Generated action.
    def activate(self, client, security):
        return client.method(self, 1, security, DataType.ENUM)

    #
    # Updates one or more global keys.
    #
    # client: DLMS client that is used to generate action.
    # kek: Master key, also known as Key Encrypting Key.
    # list: List of Global key types and keys.
    # Generated action.
    #
    def globalKeyTransfer(self, client, kek, list_):
        # pylint: disable=import-outside-toplevel
        from ..secure.GXDLMSSecureClient import GXDLMSSecureClient
        if not list_:
            raise ValueError("Invalid list. It is empty.")
        bb = GXByteBuffer()
        bb.setUInt8(DataType.ARRAY)
        bb.setUInt8(int(len(list_)))
        for it in list_:
            bb.setUInt8(DataType.STRUCTURE)
            bb.setUInt8(2)
            _GXCommon.setData(None, bb, DataType.ENUM, it.getKey())
            tmp = GXDLMSSecureClient.encrypt(kek, it)
            _GXCommon.setData(None, bb, DataType.OCTET_STRING, tmp)
        return client.method(self, 2, bb.array(), DataType.ARRAY)

    #
    # Agree on one or more symmetric keys using the key agreement
    #      algorithm.
    #
    # @param client
    # DLMS client that is used to generate action.
    # @param list
    # List of keys.
    # Generated action.
    #
    def __keyAgreement(self, client, list_):
        if not list_:
            raise ValueError("Invalid list. It is empty.")
        bb = GXByteBuffer()
        bb.setUInt8(DataType.ARRAY)
        bb.setUInt8(int(len(list_)))
        for it in list_:
            bb.setUInt8(DataType.STRUCTURE)
            bb.setUInt8(2)
            _GXCommon.setData(None, bb, DataType.ENUM, it.getKey())
            _GXCommon.setData(None, bb, DataType.OCTET_STRING, it)
        return client.method(self, 3, bb.array(), DataType.ARRAY)

    #
    # Agree on one or more symmetric keys using the key agreement
    #      algorithm.
    #
    # @param client
    # DLMS client that is used to generate action.
    # @param type
    # Global key type.
    # Generated action.
    def keyAgreement(self, client, type_):
        bb = GXByteBuffer()
        data = self.getEphemeralPublicKeyData(type_, client.ciphering.ephemeralKeyPair.public)
        bb.set(data, 1, 64)
        print("Signin public key: " + str(client.ciphering.signingKeyPair.public))
        # sign = GXASymmetric.getEphemeralPublicKeySignature(type_ ,
        # client.ciphering.ephemeralKeyPair.public,
        # client.ciphering.signingKeyPair.private)
        #  bb.set(sign)
        # print("Data: " + GXByteBuffer.hex(data))
        #print("Sign: " + GXByteBuffer.hex(sign))
        list_ = list()
        list_.append((type_, bb.array()))
        return self.__keyAgreement(client, list_)

    def generateKeyPair(self, client, type_):
        return client.method(self, 4, type_, DataType.ENUM)

    def generateCertificate(self, client, type_):
        return client.method(self, 5, type_, DataType.ENUM)


    def importCertificate(self, client, certificate):
        return self.importCertificate(client, certificate.getEncoded())

    def importCertificate_0(self, client, key):
        return client.method(self, 6, key, DataType.OCTET_STRING)

    def exportCertificateByEntity(self, client, entity, type_, systemTitle):
        if not systemTitle:
            raise ValueError("Invalid system title.")
        bb = GXByteBuffer()
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(0)
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(3)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(entity)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(type_)
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, systemTitle)
        return client.method(self, 7, bb.array(), DataType.STRUCTURE)

    def exportCertificateBySerial(self, client, serialNumber, issuer):
        bb = GXByteBuffer()
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(1)
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, serialNumber.encode())
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, issuer.encode())
        return client.method(self, 7, bb.array(), DataType.STRUCTURE)

    def removeCertificateByEntity(self, client, entity, type_, systemTitle):
        bb = GXByteBuffer()
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(0)
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(3)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(entity)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(type_)
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, systemTitle)
        return client.method(self, 8, bb.array(), DataType.STRUCTURE)

    def removeCertificateBySerial(self, client, serialNumber, issuer):
        bb = GXByteBuffer()
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        bb.setUInt8(DataType.ENUM)
        bb.setUInt8(1)
        bb.setUInt8(DataType.STRUCTURE)
        bb.setUInt8(2)
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, serialNumber.encode())
        _GXCommon.setData(None, bb, DataType.OCTET_STRING, issuer.encode())
        return client.method(self, 8, bb.array(), DataType.STRUCTURE)

    @classmethod
    def systemTitleToSubject(cls, systemTitle):
        bb = GXByteBuffer(systemTitle)
        bb.setUInt8(0, 0)
        bb.setUInt8(0, 1)
        bb.setUInt8(0, 2)
        subject = "CN=" + str(systemTitle, 0, 3)
        subject += str(bb.getUInt64())
        return subject

    def invoke(self, settings, e):
        #pylint: disable=bad-option-value,redefined-variable-type
        if e.index == 1:
            if self.version == 0:
                self.securityPolicy = SecurityPolicy0(e.parameters)
                if self.securityPolicy == SecurityPolicy0.AUTHENTICATED:
                    settings.cipher.security = Security.AUTHENTICATION
                elif self.securityPolicy == SecurityPolicy0.ENCRYPTED:
                    settings.cipher.security = Security.ENCRYPTION
                elif self.securityPolicy == SecurityPolicy0.AUTHENTICATED_ENCRYPTED:
                    settings.cipher.security = Security.AUTHENTICATION_ENCRYPTION
            elif self.version == 1:
                self.securityPolicy = SecurityPolicy(e.parameters)
                if self.securityPolicy & SecurityPolicy.AUTHENTICATED_RESPONSE != 0:
                    settings.cipher.security = Security(settings.cipher.security | Security.AUTHENTICATION)
                if self.securityPolicy & SecurityPolicy.ENCRYPTED_RESPONSE != 0:
                    settings.cipher.security = Security(settings.cipher.security | Security.ENCRYPTION)
        elif e.index == 2:
            # pylint: disable=import-outside-toplevel
            from ..secure.GXDLMSSecureClient import GXDLMSSecureClient
            # if settings.Cipher is null non secure server is used.
            # Keys are take in action after reply is generated.
            for tmp in e.parameters:
                item = tmp
                type_ = GlobalKeyType(item[0])
                if type_ == GlobalKeyType.UNICAST_ENCRYPTION:
                    GXDLMSSecureClient.decrypt(settings.kek, item[1])
                elif type_ == GlobalKeyType.BROADCAST_ENCRYPTION:
                    e.error = ErrorCode.READ_WRITE_DENIED
                elif type_ == GlobalKeyType.AUTHENTICATION:
                    GXDLMSSecureClient.decrypt(settings.kek, item[1])
                elif type_ == GlobalKeyType.KEK:
                    GXDLMSSecureClient.decrypt(settings.kek, item[1])
                else:
                    e.error = ErrorCode.READ_WRITE_DENIED
        elif e.index == 3:
            e.rror = ErrorCode.HARDWARE_FAULT
        elif e.index == 4:
            e.rror = ErrorCode.HARDWARE_FAULT
        elif e.index == 5:
            e.rror = ErrorCode.HARDWARE_FAULT
        elif e.index == 6:
            e.rror = ErrorCode.HARDWARE_FAULT
        elif e.index == 8:
            e.error = ErrorCode.READ_WRITE_DENIED
        else:
            e.error = ErrorCode.READ_WRITE_DENIED

    @classmethod
    def applyKeys(cls, settings, e):
        #pylint: disable=import-outside-toplevel
        from ..secure.GXDLMSSecureClient import GXDLMSSecureClient
        for tmp in e.parameters:
            item = tmp
            type_ = GlobalKeyType(item[0])
            data = GXDLMSSecureClient.decrypt(settings.kek, item[1])
            if type_ == GlobalKeyType.UNICAST_ENCRYPTION:
                settings.cipher.blockCipherKey = data
            elif type_ == GlobalKeyType.BROADCAST_ENCRYPTION:
                e.error = ErrorCode.READ_WRITE_DENIED
            elif type_ == GlobalKeyType.AUTHENTICATION:
                settings.cipher.authenticationKey = data
            elif type_ == GlobalKeyType.KEK:
                settings.kek = data
            else:
                e.error = ErrorCode.READ_WRITE_DENIED

    def getAttributeIndexToRead(self, all_):
        attributes = list()
        if all_ or not self.logicalName:
            attributes.append(1)
        if all_ or self.canRead(2):
            attributes.append(2)
        if all_ or self.canRead(3):
            attributes.append(3)
        if all_ or self.canRead(4):
            attributes.append(4)
        if all_ or self.canRead(5):
            attributes.append(5)
        if self.version != 0:
            if all_ or self.canRead(6):
                attributes.append(6)
        return attributes

    def getAttributeCount(self):
        if self.version == 0:
            return 5
        return 6

    def getMethodCount(self):
        if self.version == 0:
            return 2
        return 8

    def getDataType(self, index):
        if index == 1:
            ret = DataType.OCTET_STRING
        elif index == 2:
            ret = DataType.ENUM
        elif index == 3:
            ret = DataType.ENUM
        elif index == 4:
            ret = DataType.OCTET_STRING
        elif index == 5:
            ret = DataType.OCTET_STRING
        elif self.version > 0:
            if index == 6:
                ret = DataType.ARRAY
            else:
                raise ValueError("getDataType failed. Invalid attribute index.")
        else:
            raise ValueError("getDataType failed. Invalid attribute index.")
        return ret

    @classmethod
    def getCertificatesByteArray(cls, settings):
        bb = GXByteBuffer()
        bb.setUInt8(DataType.ARRAY)
        if settings.cipher.certificates:
            _GXCommon.setObjectCount(len(settings.cipher.certificates), bb)
            for it in settings.cipher.certificates:
                bb.setUInt8(DataType.STRUCTURE)
                _GXCommon.setObjectCount(6, bb)
                bb.setUInt8(DataType.ENUM)
                bb.setUInt8(CertificateEntity.SERVER)
                bb.setUInt8(DataType.ENUM)
                bb.setUInt8(CertificateType.DIGITAL_SIGNATURE)
                _GXCommon.addString(it.serialNumber, bb)
                _GXCommon.addString(it.issuer, bb)
                _GXCommon.addString(it.subject, bb)
                _GXCommon.addString("", bb)
        else:
            bb.setUInt8(0)
        return bb.array()

    def getValue(self, settings, e):
        #pylint: disable=bad-option-value,redefined-variable-type
        if e.index == 1:
            ret = _GXCommon.logicalNameToBytes(self.logicalName)
        elif e.index == 2:
            ret = self.securityPolicy
        elif e.index == 3:
            ret = self.securitySuite
        elif e.index == 4:
            ret = self.clientSystemTitle
        elif e.index == 5:
            ret = self.serverSystemTitle
        elif e.index == 6:
            ret = self.getCertificatesByteArray(settings)
        else:
            e.error = ErrorCode.READ_WRITE_DENIED
        return ret

    def updateSertificates(self, list_):
        self.certificates = []
        if list_:
            for it in list_:
                info = GXDLMSCertificateInfo()
                info.entity = CertificateEntity(it[0])
                info.type_ = CertificateType(it[1])
                info.serialNumber = it[2]
                info.issuer = it[3]
                info.subject = it[4]
                info.subjectAltName = it[5]
                self.certificates.append(info)

    def setValue(self, settings, e):
        #pylint: disable=bad-option-value,redefined-variable-type
        if e.index == 1:
            self.logicalName = _GXCommon.toLogicalName(e.value)
        elif e.index == 2:
            if settings.isServer:
                e.error = ErrorCode.READ_WRITE_DENIED
            else:
                if self.version == 0:
                    self.securityPolicy = SecurityPolicy0(e.value)
                else:
                    self.securityPolicy = SecurityPolicy(e.value)
        elif e.index == 3:
            self.securitySuite = e.value
        elif e.index == 4:
            self.clientSystemTitle = e.value
        elif e.index == 5:
            self.serverSystemTitle = e.value
        elif e.index == 6:
            self.updateSertificates(e.value)
        else:
            e.error = ErrorCode.READ_WRITE_DENIED

    @classmethod
    def getEphemeralPublicKeyData(cls, keyId, ephemeralKey):
        # pylint: disable=unused-argument
        #tmp =
        #(GXAsn1Converter.fromByteArray(ephemeralKey.getEncoded())).get(1)
        #epk = GXByteBuffer(tmp.value)
        #epk.setUInt8(int(keyId), 0)
        #return epk
        return None

    def load(self, reader):
        self.securityPolicy = SecurityPolicy(reader.readElementContentAsInt("SecurityPolicy"))
        self.securitySuite = SecuritySuite(reader.readElementContentAsInt("SecuritySuite"))
        str_ = reader.readElementContentAsString("ClientSystemTitle")
        if str_ is None:
            self.clientSystemTitle = None
        else:
            self.clientSystemTitle = GXByteBuffer.hexToBytes(str_)
        str_ = reader.readElementContentAsString("ServerSystemTitle")
        if str_ is None:
            self.serverSystemTitle = None
        else:
            self.serverSystemTitle = GXByteBuffer.hexToBytes(str_)
        self.certificates = []
        if reader.isStartElement("Certificates", True):
            while reader.isStartElement("Item", True):
                it = GXDLMSCertificateInfo()
                self.certificates.append(it)
                it.entity = CertificateEntity(reader.readElementContentAsInt("Entity"))
                it.type_ = CertificateType(reader.readElementContentAsInt("Type"))
                it.serialNumber = reader.readElementContentAsString("SerialNumber")
                it.issuer = reader.readElementContentAsString("Issuer")
                it.subject = reader.readElementContentAsString("Subject")
                it.subjectAltName = reader.readElementContentAsString("SubjectAltName")
            reader.readEndElement("Certificates")

    def save(self, writer):
        writer.writeElementString("SecurityPolicy", int(self.securityPolicy))
        writer.writeElementString("SecuritySuite", int(self.securitySuite))
        writer.writeElementString("ClientSystemTitle", GXByteBuffer.hex(self.clientSystemTitle))
        writer.writeElementString("ServerSystemTitle", GXByteBuffer.hex(self.serverSystemTitle))
        writer.writeStartElement("Certificates")
        if self.certificates:
            for it in self.certificates:
                writer.writeStartElement("Item")
                writer.writeElementString("Entity", it.entity)
                writer.writeElementString("Type", it.type_)
                writer.writeElementString("SerialNumber", it.serialNumber)
                writer.writeElementString("Issuer", it.issuer)
                writer.writeElementString("Subject", it.subject)
                writer.writeElementString("SubjectAltName", it.subjectAltName)
                writer.writeEndElement()
        writer.writeEndElement()



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename:        $HeadURL$
#
#  Version:         $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
import datetime

#  Reserved for internal use.
#pylint222: disable=bad-option-value,old-style-class,too-few-public-methods
class GXTimeZone(datetime.tzinfo):
    """
    UTC offset from UTC.

    :param offset:
        UTC time zone offset in minutes.
    """
    def __init__(self, offset):
        self._offset = datetime.timedelta(seconds=offset * 60)
        if offset == 0:
            self._name = "Z"
        else:
            if offset > 0:
                self._name = "+"
            else:
                self._name = "-"
            self._name += str(int(offset / 60)).zfill(2)
            self._name += ":"
            self._name += str(offset % 60).zfill(2)

    def utcoffset(self, dt):
        ###UTC offset in seconds.
        return self._offset

    def dst(self, dt):
        return datetime.timedelta(0)

    def tzname(self, dt):
        return self._name



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from gurux_dlms.GXIntEnum import GXIntEnum

class TranslatorTags(GXIntEnum):
    WRAPPER = 0xFF01
    HDLC = 0xFF02
    PDU_DLMS = 0xFF03
    TARGET_ADDRESS = 0xFF04
    SOURCE_ADDRESS = 0xFF05
    LIST_OF_VARIABLE_ACCESS_SPECIFICATION = 0xFF06
    LIST_OF_DATA = 0xFF07
    SUCCESS = 0xFF08
    DATA_ACCESS_ERROR = 0xFF09
    ATTRIBUTE_DESCRIPTOR = 0xFF0A
    CLASS_ID = 0xFF0B
    INSTANCE_ID = 0xFF0C
    ATTRIBUTE_ID = 0xFF0D
    METHOD_INVOCATION_PARAMETERS = 0xFF0E
    SELECTOR = 0xFF0F
    PARAMETER = 0xFF10
    LAST_BLOCK = 0xFF11
    BLOCK_NUMBER = 0xFF12
    RAW_DATA = 0xFF13
    METHOD_DESCRIPTOR = 0xFF14
    METHOD_ID = 0xFF15
    RESULT = 0xFF16
    RETURN_PARAMETERS = 0xFF17
    ACCESS_SELECTION = 0xFF18
    VALUE = 0xFF19
    ACCESS_SELECTOR = 0xFF1A
    ACCESS_PARAMETERS = 0xFF1B
    ATTRIBUTE_DESCRIPTOR_LIST = 0xFF1C
    ATTRIBUTE_DESCRIPTOR_WITH_SELECTION = 0xFF1D
    READ_DATA_BLOCK_ACCESS = 0xFF1E
    WRITE_DATA_BLOCK_ACCESS = 0xFF1F
    DATA = 0xFF20
    INVOKE_ID = 0xFF21
    DATE_TIME = 0xFF22
    REASON = 0xFF23
    VARIABLE_ACCESS_SPECIFICATION = 0xFF24
    PDU_CSE = 0xFF26
    CHOICE = 0xFF27
    LONG_INVOKE_ID = 0xFF28
    NOTIFICATION_BODY = 0xFF29
    DATA_VALUE = 0xFF30
    ACCESS_REQUEST_BODY = 0xFF31
    LIST_OF_ACCESS_REQUEST_SPECIFICATION = 0xFF32
    ACCESS_REQUEST_SPECIFICATION = 0xFF33
    ACCESS_REQUEST_LIST_OF_DATA = 0xFF34
    ACCESS_RESPONSE_BODY = 0xFF35
    LIST_OF_ACCESS_RESPONSE_SPECIFICATION = 0xFF36
    ACCESS_RESPONSE_SPECIFICATION = 0xFF37
    ACCESS_RESPONSE_LIST_OF_DATA = 0xFF38
    SINGLE_RESPONSE = 0xFF39
    SERVICE = 0xFF40
    SERVICE_ERROR = 0xFF41
    INITIATE_ERROR = 0xFF42
    CIPHERED_SERVICE = 0xFF43
    SYSTEM_TITLE = 0xFF44
    DATA_BLOCK = 0xFF45
    TRANSACTION_ID = 0xFF46
    ORIGINATOR_SYSTEM_TITLE = 0xFF47
    RECIPIENT_SYSTEM_TITLE = 0xFF48
    OTHER_INFORMATION = 0xFF49
    KEY_INFO = 0xFF50
    AGREED_KEY = 0xFF51
    KEY_PARAMETERS = 0xFF52
    KEY_CIPHERED_DATA = 0xFF53
    CIPHERED_CONTENT = 0xFF54
    ATTRIBUTE_VALUE = 0xFF55
    CURRENT_TIME = 0xFF56
    TIME = 0xFF57
    MAX_INFO_RX = 0xFF58
    MAX_INFO_TX = 0xFF59
    WINDOW_SIZE_RX = 0xFF60
    WINDOW_SIZE_TX = 0xFF61
    VALUE_LIST = 0xFF62
    DATA_ACCESS_RESULT = 0xFF63
    FRAME_TYPE = 0xFF64
    BLOCK_CONTROL = 0xFF65
    BLOCK_NUMBER_ACK = 0xFF66
    BLOCK_DATA = 0xFF67
    CONTENTS_DESCRIPTION = 0xFF68
    ARRAY_CONTENTS = 0xFF69
    NETWORK_ID = 0xFF7A
    PHYSICAL_DEVICE_ADDRESS = 0xFF7B
    PROTOCOL_VERSION = 0xFF7C
    CALLED_AP_TITLE = 0xFF7D
    CALLED_AP_INVOCATION_ID = 0xFF7E
    CALLED_AE_INVOCATION_ID = 0xFF7F
    CALLING_AP_INVOCATION_ID = 0xFF80
    CALLED_AE_QUALIFIER = 0xFF81
    RESPONSE_ALLOWED = 0xFF82
    EXCEPTION_RESPONSE = 0xFF83
    STATE_ERROR = 0xFF84
